%PSDATAGEN: Synthetic dataset generator. By setting the parameters one can
%   obtain datasets which model different configurations.

%   E. Crabu & G. Rodriguez
%   Email: elisa.crabu@unica.it, rodriguez@unica.it
%
%   Last revised August, 2024

% Initialization of the parameters
fsiz = 12;		% font size for figures
displayfactor = 1;	% for displaying images on screen
maxcolor = 1; 		% set the max intensity of colors in graphs
nrow = 3; ncol = 4;	% size of grid for figure 2

A = 2;			% horizontal size of image in units of length
funind = 2;		% test function index
albedotype = 1;		% test albedo index
r = 101;		% horizontal size of image in pixels
s = 101;		% vertical size of image in pixels

% Choose the configuration of the problem.

% Function which represents the surface of the synthetic object
surftype = 1;		% 1 for symbolic, 2 for discrete surface
surface = struct;	% create an empty struct
switch surftype
case 1		% symbolic surface
	[f,fx,fy,delta] = choosefun(funind);
	surface.f = f;
	surface.fx = fx;
	surface.fy = fy;
case 2		% discretized surface
	[X Y Z] = peaks; Z = Z/4;
	surface.X = X;
	surface.Y = Y;
	surface.Z = Z;
	[s,r] = size(Z);
otherwise
	error('Unknown surface type.')
end

% Distance of the lights in units of length
ray = 0;		% for lights at infinite distance
%ray = 4*A;		% for lights at a chosen close distance, compared to A

% Reflection
reflectau = 0;		% no reflection 
%reflectau = [1e-1,5];	% [tau,reflfact] tolerance and reflection factor

% Set angles for lights:
% angles(1) is the light at east, angles(light2) is the light at west.
% Having two known lights is useful as a reference for reconstruction.
% Setting light2=0 disable this feature.
%angles = [0:pi/2:3/2*pi]; light2 = 3;				% 4 lights
%angles = [0 pi/4 3/4*pi pi 5/4*pi 3/2*pi]; light2 = 4;		% 6 lights
%angles = [0 pi/4 3/4*pi pi 5/4*pi 3/2*pi 7/4*pi]; light2 = 4; 	% 7 lights
%angles = [0:pi/4:7/4*pi]; light2 = 5;				% 8 lights
angles = [0:pi/6:11/6*pi]; light2 = 7;				% 12 lights

% Set z coordinate for lights
% notice that if angles are equispaced and z is the same for all lights, the
% lights matrix L is rank deficient.
z = [ones(1,numel(angles)-1),1.5];

% Perturbate lights position, except for two reference lights.
ds = 0;		% no perturbation
%ds = 0.1;
noise = ds*randn(1,numel(angles));
if light2>0, noise([1 light2]) = 0; end

% Construct lights matrix
L = createlights(angles+noise,z+noise);

% Compute constants
p = r*s;		% pixels number
q = size(L,2);		% lights number

% If lights are at a finite distance, store L in homogeneous coordinates 
if ray
	Lh = [L;1/ray*ones(1,q)];
else
	% Lights at infinite distance
	Lh = L;
end

fprintf('\nPhotometric Stereo synthetic data generation.\n\n')
fprintf('images size  : %d X %d\n',r,s)
fprintf('problem size : %d\n',p)
fprintf('images number: %d\n',q)

% Create the albedo
D = makealbedo(albedotype,r,s);

% Create the synthetic data matrix and the surface of the object
[M,X,Y,Z,N,h,B] = makepsimages(surface,Lh,A,r,s,D,reflectau);

% Open and clear figures
for i = 1:4, figure(i), clf, end

% Plot the surface
figure(1)
surfl(X,Y,Z)
colormap gray
shading interp
axis equal
set(gca,'fontsize',fsiz)
title('Model surface','fontsize',fsiz)

% Plot light directions
figure(2)
plotlights3d(L,2,fsiz)
title('Light directions','fontsize',fsiz)

% Images of the surface obtained by q chosen lighting directions
figure(3)
for i = 1:q
	subplot(nrow,ncol,i)
	psimshow(squeeze(M(:,i,:)),s,r,displayfactor,maxcolor)
	if i==1, title('Synthetic dataset','fontsize',fsiz), end
end

if size(reflectau,2)>1
	% Create data without reflections
	Mnr = makepsimages(surface,Lh,A,r,s,D);
	% Plot the difference between images with and without reflection
	figure(4)
	for i = 1:q
		figure(4)
		subplot(nrow,ncol,i)
		psimshow(squeeze(M(:,i,:)-Mnr(:,i,:)),s,r,displayfactor)
		if i==1, title('Reflection points','fontsize',fsiz), end
	end
end

fprintf('\nAvailable variables')
fprintf('\nPS data matrix: M\n')
fprintf('Normal vectors: N, Lights: L, Albedo: D\n')
fprintf('Lights in homogeneous coordinates: Lh\n')
fprintf('Surface data: X, Y, Z\n')
fprintf('Images size (pixels): r, s\n')
fprintf('Physical size of images and pixels: A, B, h\n')

